'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /* global FormData */

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _formData = require('form-data');

var _formData2 = _interopRequireDefault(_formData);

var _urlSearchParams = require('url-search-params');

var _urlSearchParams2 = _interopRequireDefault(_urlSearchParams);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MIME_TYPE_FORM = 'application/x-www-form-urlencoded';
var MIME_TYPE_JSON = 'application/json';
var MIME_TYPE_MULTIPART_FORM_DATA = 'multipart/form-data';

function createResponse(axiosResponse) {
  if ((typeof axiosResponse === 'undefined' ? 'undefined' : _typeof(axiosResponse)) !== 'object') {
    return axiosResponse;
  }

  return {
    status: axiosResponse.status,
    statusText: axiosResponse.statusText,
    headers: axiosResponse.headers,
    data: axiosResponse.data
  };
}

function createError(axiosError) {
  return {
    name: axiosError.name,
    code: axiosError.code,
    message: axiosError.message,
    response: createResponse(axiosError.response)
  };
}

_axios2.default.interceptors.response.use(function (response) {
  return createResponse(response);
}, function (error) {
  return Promise.reject(createError(error));
});

var methods = {
  create: 'POST',
  read: 'GET',
  update: 'PUT',
  delete: 'DELETE'
};

function toFormData(data) {
  var params = new _urlSearchParams2.default();

  if (data) {
    Object.keys(data).forEach(function (key) {
      return params.set(key, data[key]);
    });
  }

  return params.toString();
}

function toMultipartFormData(data) {
  var formData = typeof FormData !== 'undefined' ? new FormData() : new _formData2.default();

  if (data) {
    Object.keys(data).forEach(function (key) {
      return formData.append(key, data[key]);
    });
  }

  return formData;
}

function getConfig(requestConfig) {
  var url = requestConfig.resource;
  var method = methods[requestConfig.operation];
  var baseURL = requestConfig.url;
  var params = requestConfig.params;
  var responseType = requestConfig.responseType;

  var data = null;

  if (requestConfig.type === MIME_TYPE_FORM) {
    data = toFormData(requestConfig.data);
  } else if (requestConfig.type === MIME_TYPE_MULTIPART_FORM_DATA) {
    data = toMultipartFormData(requestConfig.data);
  } else {
    data = requestConfig.data;
  }

  var headers = {};

  var auth = null;

  if (_config2.default.hasOption(requestConfig, 'token')) {
    headers.Authorization = 'Bearer ' + requestConfig.token;
  } else if (requestConfig.username || requestConfig.password) {
    auth = {
      username: requestConfig.username,
      password: requestConfig.password
    };
  }

  if (_config2.default.hasOption(requestConfig, 'type')) {
    var type = requestConfig.type;

    if (type === MIME_TYPE_FORM || type === MIME_TYPE_JSON) {
      headers['Content-Type'] = type + ';charset=utf-8';
    } else if (type === MIME_TYPE_MULTIPART_FORM_DATA) {
      if (data && data.getHeaders) {
        // If we're in node, using form-data, axios won't natively know what to do (it just sees a
        // stream). We need to manually set the content type header to have the correct mime type
        // and boundary, using the getHeaders method. If we're in a browser, using the built-in
        // FormData, the header will be set automatically, so we don't have to do anything.
        headers['Content-Type'] = data.getHeaders()['content-type'];
      }
    } else {
      headers['Content-Type'] = type;
    }
  }

  return {
    url: url,
    method: method,
    baseURL: baseURL,
    headers: headers,
    params: params,
    data: data,
    auth: auth,
    responseType: responseType
  };
}

function sendRequest(requestConfig) {
  return (0, _axios2.default)(getConfig(requestConfig));
}

exports.default = {
  getConfig: getConfig,
  sendRequest: sendRequest
};